﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows; 
using Infragistics.Controls.Charts;

namespace XamDataChart_StackColumnSeries
{
    /// <summary>
    /// MainWindow.xaml  
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            CreateStackedFragmentMarkers();
        }

        private void CreateStackedFragmentMarkers()
        {
            var stackedColumnSeries = this.DataChart.Series.OfType<StackedColumnSeries>().First();
            var stackedDataSource = stackedColumnSeries.ItemsSource; // as EnergyProductionData;
              
            var dataColumnNames = new List<string>();
            var categoryAxis = stackedColumnSeries.XAxis as CategoryXAxis;

            var vm = new StackedSeriesViewModel();
            if (categoryAxis != null && stackedDataSource != null)
            {
                foreach (var segment in stackedColumnSeries.Series)
                {
                    dataColumnNames.Add(segment.ValueMemberPath);
                }
                var propCategoryName = categoryAxis.Label.ToString().Replace("{", "").Replace("}", "");
                foreach (var item in stackedDataSource)
                {
                    foreach (var propName in dataColumnNames)
                    {
                        vm.Add(item, propCategoryName, propName);
                    }
                }
            }
             
            var markerSeries = this.DataChart.Series.OfType<StackedLineSeries>().First();
          
            markerSeries.ItemsSource = vm.StackGroups;
            
            foreach (var segment in stackedColumnSeries.Series)
            {
                var markerSegment = new StackedFragmentSeries();
                markerSegment.Title = segment.ValueMemberPath;
                markerSegment.ValueMemberPath = "Center." + segment.ValueMemberPath;
                markerSegment.MarkerTemplate = this.Resources[segment.ValueMemberPath + "Template"] as DataTemplate;
                markerSeries.Series.Add(markerSegment);
            }
           
        }
        public class StackedSeriesViewModel
        {
            public StackedSeriesViewModel()
            {
                Stacks = new Dictionary<string, StackedFragmentGroup>();
                StackGroupNames = new List<string>();
                StackSegmentNames = new List<string>(); 
            }

            #region Properties
            public Dictionary<string, StackedFragmentGroup> Stacks { get; private set; }
            public List<string> StackGroupNames { get; private set; }
            public List<string> StackSegmentNames { get; private set; }

            /// <summary>
            /// Gets a list of stacked fragments with specified fragment name  
            /// </summary>
            public List<StackedFragment> StackFragments(string fragmentName)
            {
                var list = new List<StackedFragment>();
                foreach (var stack in Stacks.Values)
                {
                    if (stack.Contains(fragmentName))
                    {
                        list.Add(stack[fragmentName]);
                    }
                }
                return list;
            }

            /// <summary>
            /// Gets a list of stacked fragments with specified fragment name  
            /// </summary>
            public List<StackedFragment> this[string fragmentName]
            {
                get { return StackFragments(fragmentName); }
            }

            /// <summary>
            /// Gets a list of stacked groups  
            /// </summary>
            public List<StackedFragmentGroup> StackGroups
            {
                get
                {
                    var list = new List<StackedFragmentGroup>();
                    foreach (var stack in Stacks.Values)
                    {
                        list.Add(stack);
                    }
                    return list;
                }
            }

            #endregion
            public void Add(object item, string categoryMemberPath, string valueMemberPath)
            {
                if (item == null) return;

                var propCategory = item.GetPropertyValue(categoryMemberPath).ToString();
                var propValue = 0.0;
                if (double.TryParse(item.GetPropertyValue(valueMemberPath).ToString(), out propValue))
                {
                    var fragment = new StackedFragment();
                    fragment.Data = item;
                    fragment.Name = valueMemberPath;
                    fragment.Category = propCategory;
                    fragment.Value = propValue;

                    if (Stacks.ContainsKey(propCategory))
                    {
                        Stacks[propCategory].Add(fragment);
                    }
                    else
                    {
                        var group = new StackedFragmentGroup();
                        group.Add(fragment);
                        group.Category = propCategory;
                        Stacks.Add(propCategory, group);
                    }

                    if (!StackGroupNames.Contains(propCategory))
                    {
                        StackGroupNames.Add(propCategory);
                    }
                    if (!StackSegmentNames.Contains(valueMemberPath))
                    {
                        StackSegmentNames.Add(valueMemberPath);
                    }
                   

                }
            }
        }
     

        public class StackedFragment
        {
            /// <summary> Gets or sets original data associated with StackedFragment </summary>
            public object Data { get; set; }

            /// <summary> Gets or sets Center of StackedFragment </summary>
            public double Center { get; set; }

            /// <summary> Gets or sets Value of StackedFragment </summary>
            public double Value { get; set; }

            public string Category { get; set; }
            public string Name { get; set; }
        }
        public class StackedFragmentDictionary : Dictionary<string, StackedFragment>
        {

        }

        public class StackedFragmentGroup  
        {
            /// <summary> Gets or sets original data associated with StackedFragment </summary>
            public object Data { get; set; }
            public object Center { get; set; }
            public string Category { get; set; }
  
            public double StackValue { get; set; }
            protected StackedFragmentDictionary StackFragments { get; private set; }
            
            public StackedFragmentGroup()
            {
                StackValue = 0;
                StackFragments = new StackedFragmentDictionary();
            }

            public StackedFragment this[string fragmentName]
            {
                get { return StackFragments[fragmentName]; }
                set { StackFragments[fragmentName] = value; }
            }

            protected double StackPreviousCenter;

            public bool Contains(StackedFragment fragment)
            {
                return Contains(fragment.Name);
            }
            public bool Contains(string fragmentName)
            {
                return StackFragments.ContainsKey(fragmentName);
            }
            public void Add(StackedFragment fragment)
            {
                if (Data == null)
                {
                    Data = fragment.Data;
                    Center = fragment.Data.Clone();
                }
                 
                var fragmentCenter = fragment.Value / 2;
                var fragmentStackCenter = StackValue + fragmentCenter;

                Center.SetPropertyValue(fragment.Name, StackPreviousCenter + fragmentCenter);
                fragment.Center = fragmentStackCenter;

                StackPreviousCenter = fragmentCenter;
                StackValue += fragment.Value;
           
                if (StackFragments.ContainsKey(fragment.Name))
                {
                    StackFragments[fragment.Name] = fragment;
                }
                else
                {
                    StackFragments.Add(fragment.Name, fragment);
                }
            }

            public void Remove(string fragmentName)
            {
                StackValue = 0;
                StackFragments.Remove(fragmentName);
                foreach (var item in StackFragments.Values)
                {
                    item.Center = StackValue + (item.Value / 2);
                    StackValue += item.Value;
                }
            }
            public void Remove(StackedFragment fragment)
            {
                Remove(fragment.Name);
            }
            public void Clear()
            {
                StackFragments.Clear();
                StackValue = 0;
            }
        
        }

      
       
       
    }

    

}
