﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Windows;

namespace System
{
    public static class ObjectEx
    {
        /// <summary>
        /// Gets value of specified property name
        /// </summary>
        public static object GetPropertyValue(this object src, string propName)
        {
            return src.GetType().GetProperty(propName).GetValue(src, null);
        }
        /// <summary>
        /// Sets value of specified property name
        /// </summary>
        public static bool SetPropertyValue(this object obj, string propName, object propValue)
        {
            var prop = obj.GetType().GetProperty(propName, BindingFlags.Public | BindingFlags.Instance);
            if (null != prop && prop.CanWrite)
            {
                prop.SetValue(obj, propValue, null);
                return true;
            }
            return false;
        }
        /// <summary>
        /// Clones current object
        /// </summary>
        public static object Clone(this object src)
        {
            var type = src.GetType();
            var clone = Activator.CreateInstance(type, new object[] {});

            var properties = type.GetProperties().ToList();
          
            foreach (var property in properties)
            {
                if (property.CanWrite)
                {
                    var propValue = src.GetPropertyValue(property.Name);
                    clone.SetPropertyValue(property.Name, propValue);
                }
            }
            return clone;
        }
        
     
    }
    public static class TypeEx
    {
        /// <summary>
        /// Gets property fields of the current type and its base types with optional dependency properties
        /// </summary>
        public static List<FieldInfo> GetPropertyFields(this Type type, bool includeBaseType, bool isDependency)
        {
            var fields = type.GetPropertyFields(includeBaseType).ToList();
            if (isDependency)
            {
                fields = fields.Where(p => p.FieldType == typeof(DependencyProperty)).ToList();
            }
            return fields;
        }

        /// <summary>
        /// Gets property fields of the current type and its base types
        /// </summary>
        public static List<FieldInfo> GetPropertyFields(this Type type, bool includeBaseType = true)
        {
            var fields = type.GetFields(BindingFlags.Static | BindingFlags.Public).ToList();
            if (includeBaseType && type.BaseType != null)
                fields = fields.Union(type.BaseType.GetPropertyFields()).ToList();
            return fields;
        }
         

    }


    public static class ObjectCopier
    {
        /// <summary>
        /// Perform a deep Copy of the object.
        /// </summary>
        /// <typeparam name="T">The type of object being copied.</typeparam>
        /// <param name="source">The object instance to copy.</param>
        /// <returns>The copied object.</returns>
        public static T Clone<T>(T source)
        {
            if (!typeof(T).IsSerializable)
            {
                throw new Exception("The type of source object must be serializable.");
            }

            // Don't serialize a null object, simply return the default for that object
            if (Object.ReferenceEquals(source, null))
            {
                return default(T);
            }

            IFormatter formatter = new BinaryFormatter();
            Stream stream = new MemoryStream();
            using (stream)
            {
                formatter.Serialize(stream, source);
                stream.Seek(0, SeekOrigin.Begin);
                return (T)formatter.Deserialize(stream);
            }
        }
    }
}

namespace System.Collections.Generic
{
    public static class EnumerableEx
    {
        public static List<T> ToListSafe<T>(this IEnumerable<T> obj)
        {
            return obj == null ? new List<T>() : obj.ToList();
        }
    }
}
